#
# Revisioner
#
# Copyright (c) 2001-2003 by Philipp "philiKON" von Weitershausen
#
# This software is distributed under the terms of the Mozilla Public
# License (MPL)
#

"""
$Id: Revisioner.py,v 1.4 2003/05/20 21:08:00 philipp Exp $
"""

from Acquisition import aq_acquire
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.RevisionManager import Permissions

class Revisioner:
    """
    This class is actually never instanciated. We just use it to rip
    out the methods and views later.
    """

    names = ['_rm_getRM', 'rm_getRelativePath', 'rm_getSettings',
             'rm_getStatus', 'rm_getDiff', 'rm_getLog', 'rm_add',
             'rm_remove', 'rm_updateStatus', 'rm_update',
             'rm_forceCheckout', 'rm_commit', 'rm_versioningForm',
             'rm_diffForm', 'rm_logForm']

    def _haveRevisionManager(self):
        try:
            aq_acquire(self, "RevisionManager")
            return 1
        except AttributeError:
            return 0

    manage_options = (
        {'label': 'Versioning',
         'action': 'rm_versioningForm',
         'filter': _haveRevisionManager },
        )

    permissions = (
        (Permissions.RMManage,
         ('rm_getRelativePath', 'rm_getStatus', 'rm_getDiff',
          'rm_getLog', 'rm_updateStatus', 'rm_versioningForm',
          'rm_diffForm', 'rm_logForm')),
        (Permissions.RMUpdate,
         ('rm_update', 'rm_forceCheckout')),
        (Permissions.RMAddRemove,
         ('rm_add', 'rm_remove')),
        (Permissions.RMCommit,
         ('rm_commit'))
        )

    def _rm_getRM(self):
        try:
            return aq_acquire(self, "RevisionManager")
        except AttributeError:
            return None

    def rm_getRelativePath(self):
        rm = self._rm_getRM()
        if rm is None:
            return ''
        phys_path = self.getPhysicalPath()
        rm_path = rm.getPhysicalPath()
        i = 0
        while rm_path[i] == phys_path[i]:
            i += 1
        return "/".join(phys_path[i:])

    def rm_getSettings(self):
        rm = self._rm_getRM()
        return rm.getSettings()

    def rm_getStatus(self):
        rm = self._rm_getRM()
        if rm is None:
            return {}
        path = self.rm_getRelativePath()
        try:
            return rm.getStatus(path)
        except KeyError:
            return {}

    def rm_getDiff(self):
        rm = self._rm_getRM()
        if rm is None:
            return ''
        path = self.rm_getRelativePath()
        return rm.diff([path])

    def rm_getLog(self):
        rm = self._rm_getRM()
        if rm is None:
            return {}
        path = self.rm_getRelativePath()
        return rm.log([path])

    # management views

    rm_versioningForm = PageTemplateFile("www/versioning.pt", globals())
    rm_diffForm = PageTemplateFile("www/diff.pt", globals())
    rm_logForm = PageTemplateFile("www/log.pt", globals())

    def rm_add(self, REQUEST, RESPONSE):
        """
        Add to the repository
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.add([path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')

    def rm_remove(self, REQUEST, RESPONSE):
        """
        Remove from repository
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.remove([path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')

    def rm_updateStatus(self, REQUEST, RESPONSE):
        """
        Update status
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.status([path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')

    def rm_update(self, REQUEST, RESPONSE):
        """
        Update
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.update([path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')

    def rm_commit(self, REQUEST, RESPONSE, log_message):
        """
        Commit
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.commit(log_message, [path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')

    def rm_forceCheckout(self, REQUEST, RESPONSE):
        """
        Force checkout
        """
        rm = self._rm_getRM()
        if rm is None:
            return
        path = self.rm_getRelativePath()
        rm.forceCheckout([path])
        RESPONSE.redirect(self.absolute_url() + '/rm_versioningForm')
